import random
import json
import os
from datetime import datetime, timedelta

# Beacon configuration
beacon_configs = [
    {"url": "beacon1.example.com", "jitter": 2, "interval": 300},
    {"url": "beacon2.example.com", "jitter": 5, "interval": 300}, 
    {"url": "beacon3.example.com", "jitter": 120, "interval": 300},
    {"url": "beacon4.example.com", "jitter": 150, "interval": 300},
    {"url": "beacon5.example.com", "jitter": 5, "interval": 10},
    {"url": "beacon6.example.com", "jitter": 2, "interval": 10},
    {"url": "beacon7.example.com", "jitter": 5, "interval": 20},
    {"url": "beacon8.example.com", "jitter": 10, "interval": 20},
    {"url": "beacon9.example.com", "jitter": 5, "interval": 30},
    {"url": "beacon10.example.com", "jitter": 5, "interval": 40},
    {"url": "beacon11.example.com", "jitter": 5, "interval": 50},
    {"url": "beacon12.example.com", "jitter": 10, "interval": 60},
    {"url": "beacon13.example.com", "jitter": 10, "interval": 120}

]

# Function to generate log entries
def generate_log_entries():
    start_time = datetime(2023, 8, 1, 0, 0, 0)
    end_time = datetime(2023, 8, 1, 23, 59, 59)
    logs = []

    # Iterate over each beacon config
    for config in beacon_configs:
        url = config["url"]
        jitter = config["jitter"]
        interval = config["interval"]
        
        current_time = start_time
        while current_time <= end_time:
            # Generate logs for a 2-hour beaconing period
            beacon_period_end = current_time + timedelta(hours=2)
            while current_time < beacon_period_end and current_time <= end_time:
                # Calculate the jitter
                jitter_time = random.randint(-jitter, jitter)
                log_time = current_time + timedelta(seconds=jitter_time)

                # Format the log date in ISO 8601 format
                logdate = log_time.strftime("%Y-%m-%dT%H:%M:%S.") + f"{log_time.microsecond:03d}Z"
                
                # Append the log entry to the list
                logs.append({"logdate": logdate, "url_hostname": url, "user": "-"})

                # Increment the time by the interval
                current_time += timedelta(seconds=interval)

            # Stop for 3 hours before the next beaconing period
            current_time = beacon_period_end + timedelta(hours=3)
    
    return logs

# Generate the log entries
log_entries = generate_log_entries()

# Specify the path to save the file
output_path = r"C:\Allianz\4\0211\1201\0301\beacon_logs.json"

# Ensure the directory exists
os.makedirs(os.path.dirname(output_path), exist_ok=True)

# Save the log entries as newline-delimited JSON (one per line)
with open(output_path, "w") as f:
    for entry in log_entries:
        f.write(json.dumps(entry) + "\n")

print(f"Beacon logs generated and saved to {output_path}.")
